@@ -2,11 +2,11 @@ package Kafka::Consumer;
 
 =head1 NAME
 
-Kafka::Consumer - Perl interface for 'consumer' client.
+Kafka::Consumer - Perl interface for Kafka consumer client
 
 =head1 VERSION
 
-This documentation refers to C<Kafka::Consumer> version 0.800_3 .
+This documentation refers to C<Kafka::Consumer> version 0.800_4 .
 
 =cut
 
@@ -18,7 +18,7 @@ use warnings;
 
 # ENVIRONMENT ------------------------------------------------------------------
 
-our $VERSION = '0.800_3';
+our $VERSION = '0.800_4';
 
 #-- load the modules -----------------------------------------------------------
 
@@ -42,26 +42,18 @@ use Kafka qw(
     %ERROR
     $ERROR_COMPRESSED_PAYLOAD
     $ERROR_MISMATCH_ARGUMENT
-    $ERROR_NO_ERROR
     $ERROR_PARTITION_DOES_NOT_MATCH
     $ERROR_TOPIC_DOES_NOT_MATCH
     $MESSAGE_SIZE_OVERHEAD
     $MIN_BYTES_RESPOND_IMMEDIATELY
     $RECEIVE_EARLIEST_OFFSETS
 );
+use Kafka::Exceptions;
 use Kafka::Internals qw(
     $APIKEY_FETCH
     $APIKEY_OFFSET
-    $DEFAULT_RAISE_ERROR
     _get_CorrelationId
-    last_error
-    last_errorcode
-    RaiseError
-    _fulfill_request
-    _error
     _isbig
-    _connection_error
-    _set_error
 );
 use Kafka::Connection;
 use Kafka::Message;
@@ -76,6 +68,11 @@ if ( !$BITS64 ) { eval 'use Kafka::Int64; 1;' or die "Cannot load Kafka::Int64 :
     use strict;
     use warnings;
 
+    use Scalar::Util qw(
+        blessed
+    );
+    use Try::Tiny;
+
     use Kafka qw(
         $DEFAULT_MAX_BYTES
         $DEFAULT_MAX_NUMBER_OF_OFFSETS
@@ -84,48 +81,62 @@ if ( !$BITS64 ) { eval 'use Kafka::Int64; 1;' or die "Cannot load Kafka::Int64 :
     use Kafka::Connection;
     use Kafka::Consumer;
 
-    #-- Connection
-    my $connect = Kafka::Connection->new( host => 'localhost' );
+    my ( $connection, $consumer );
+    try {
 
-    #-- Consumer
-    my $consumer = Kafka::Consumer->new( Connection  => $connect );
+        #-- Connection
+        $connection = Kafka::Connection->new( host => 'localhost' );
 
-    # Get a list of valid offsets up max_number before the given time
-    my $offsets = $consumer->offsets(
-        'mytopic',                      # topic
-        0,                              # partition
-        $RECEIVE_EARLIEST_OFFSETS,      # time
-        $DEFAULT_MAX_NUMBER_OF_OFFSETS  # max_number
-    );
-    if( $offsets ) {
-        say "Received offset: $_" foreach @$offsets;
-    }
-    say STDERR 'Error: (', $consumer->last_errorcode, ') ', $consumer->last_error
-        if !$offsets || $consumer->last_errorcode;
+        #-- Consumer
+        $consumer = Kafka::Consumer->new( Connection  => $connection );
 
-    # Consuming messages
-    my $messages = $consumer->fetch(
-        'mytopic',                      # topic
-        0,                              # partition
-        0,                              # offset
-        $DEFAULT_MAX_BYTES              # Maximum size of MESSAGE(s) to receive
-    );
-    if ( $messages ) {
-        foreach my $message ( @$messages ) {
-            if( $message->valid ) {
-                say 'payload    : ', $message->payload;
-                say 'key        : ', $message->key;
-                say 'offset     : ', $message->offset;
-                say 'next_offset: ', $message->next_offset;
-            }
-            else {
-                say 'error      : ', $message->error;
+        # Get a list of valid offsets up max_number before the given time
+        my $offsets = $consumer->offsets(
+            'mytopic',                      # topic
+            0,                              # partition
+            $RECEIVE_EARLIEST_OFFSETS,      # time
+            $DEFAULT_MAX_NUMBER_OF_OFFSETS  # max_number
+        );
+
+        if( @$offsets ) {
+            say "Received offset: $_" foreach @$offsets;
+        } else {
+            warn "Error: Offsets are not received\n";
+        }
+
+        # Consuming messages
+        my $messages = $consumer->fetch(
+            'mytopic',                      # topic
+            0,                              # partition
+            0,                              # offset
+            $DEFAULT_MAX_BYTES              # Maximum size of MESSAGE(s) to receive
+        );
+
+        if ( $messages ) {
+            foreach my $message ( @$messages ) {
+                if( $message->valid ) {
+                    say 'payload    : ', $message->payload;
+                    say 'key        : ', $message->key;
+                    say 'offset     : ', $message->offset;
+                    say 'next_offset: ', $message->next_offset;
+                } else {
+                    say 'error      : ', $message->error;
+                }
             }
         }
-    }
+
+    } catch {
+        if ( blessed( $_ ) && $_->isa( 'Kafka::Exception' ) ) {
+            warn 'Error: (', $_->code, ') ',  $_->message, "\n";
+            exit;
+        } else {
+            die $_;
+        }
+    };
 
     # Closes the consumer and cleans up
     undef $consumer;
+    undef $connection;
 
 =head1 DESCRIPTION
 
@@ -137,7 +148,11 @@ The main features of the C<Kafka::Consumer> class are:
 
 =item *
 
-Provides an object oriented model of communication.
+Provides an object-oriented API for consuming messages.
+
+=item *
+
+Provides Kafka FETCH and OFFSETS requests (FETCH does not support compression codec).
 
 =item *
 
@@ -145,64 +160,37 @@ Supports parsing the Apache Kafka 0.8 Wire Format protocol.
 
 =item *
 
-Supports Apache Kafka Requests and Responses (FETCH with
-no compression codec attribute now). Within this module we currently support
-access to FETCH, OFFSETS Requests and Responses.
-
-=item *
-
-Support for working with 64 bit elements of the Kafka Wire Format protocol
+Works with 64-bit elements of the Kafka Wire Format protocol
 on 32 bit systems.
 
 =back
 
-The Kafka consumer response has an ARRAY reference type for C<offsets> and
+The Kafka consumer response returns ARRAY references for C<offsets> and
 C<fetch> methods.
-For the C<offsets> response array has the offset integers.
 
-For the C<fetch> response the array has the class name
-L<Kafka::Message|Kafka::Message> elements.
+Array returned by C<offsets> contains offset integers.
+
+Array returned by C<fetch> contains objects of L<Kafka::Message|Kafka::Message> class.
 
 =cut
 
-our $_package_error;
-
 #-- constructor ----------------------------------------------------------------
 
 =head2 CONSTRUCTOR
 
 =head3 C<new>
 
-Creates new consumer client object. Returns the created C<Kafka::Consumer>
-object.
+Creates a new consumer client object. Returns the created C<Kafka::Consumer> object.
 
-An error will cause the program to halt or the constructor will return the
-C<Kafka::Consumer> object without halt, depending on the value of the C<RaiseError> attribute.
-
-You can use the methods of the C<Kafka::Consumer> class - L</last_errorcode>
-and L</last_error> for information about the error.
-
-C<new()> takes arguments in key-value pairs.
-The following arguments are currently recognized:
+C<new()> takes arguments in key-value pairs. The following arguments are recognized:
 
 =over 3
 
-=item C<Connection =E<gt> $connect>
+=item C<Connection =E<gt> $connection>
 
-C<$connect> is the L<Kafka::Connection|Kafka::Connection> object that allow you to communicate to
+C<$connection> is the L<Kafka::Connection|Kafka::Connection> object responsible for communication with
 the Apache Kafka cluster.
 
-=item C<RaiseError =E<gt> $mode>
-
-Optional, default is 0.
-
-An error will cause the program to halt if L</RaiseError> is true: C<confess>
-if the argument is not valid or C<die> in the other error case
-(this can always be trapped with C<eval>).
-
-You should always check for errors, when not establishing the C<RaiseError>
-mode to true.
-
 =item C<CorrelationId =E<gt> $correlation_id>
 
 Optional, default is C<undef>.
@@ -211,16 +199,16 @@ C<Correlation> is a user-supplied integer.
 It will be passed back in the response by the server, unmodified.
 The C<$correlation_id> should be an integer number.
 
-Error is thrown if C<CorrelationId> of request will not match C<CorrelationId> in response.
+C<CorrelationId> will be auto-assigned (random negative number) if it was not provided on
+creation of L<Kafka::Producer|Kafka::Producer> object.
 
-C<CorrelationId> will be auto-assigned (random negative number) if it was not provided during
-creation of C<Kafka::Producer> object.
+An exception is thrown if C<CorrelationId> sent with request does not match C<CorrelationId> received in response.
 
 =item C<ClientId =E<gt> $client_id>
 
 This is a user supplied identifier (string) for the client application.
 
-C<ClientId> will be auto-assigned if not passed in when creating C<Kafka::Producer> object.
+C<ClientId> will be auto-assigned if not passed in when creating L<Kafka::Producer|Kafka::Producer> object.
 
 =item C<MaxWaitTime =E<gt> $max_time>
 
@@ -236,11 +224,11 @@ The C<$max_time> must be a positive integer.
 
 =item C<MinBytes =E<gt> $min_bytes>
 
-RTFM: The minimum number of bytes of messages that must be available to give a response.
+The minimum number of bytes of messages that must be available to give a response.
 If the client sets this to C<$MIN_BYTES_RESPOND_IMMEDIATELY> the server will always respond
 immediately. If it is set to C<$MIN_BYTES_RESPOND_HAS_DATA>, the server will respond as soon
 as at least one partition has at least 1 byte of data or the specified timeout occurs.
-Setting higher values in combination with the bigger timeouts results in reading larger chunks of data.
+Setting higher values in combination with the bigger timeouts allows reading larger chunks of data.
 
 Optional, default is C<$MIN_BYTES_RESPOND_IMMEDIATELY>.
 
@@ -256,14 +244,14 @@ The maximum bytes to include in the message set for this partition.
 Optional, default = C<$DEFAULT_MAX_BYTES> (1_000_000).
 
 The C<$max_bytes> must be more than C<$MESSAGE_SIZE_OVERHEAD>
-(size of protocol overhead - data added by protocol for each message).
+(size of protocol overhead - data added by Kafka wire protocol to each message).
 
 C<$DEFAULT_MAX_BYTES>, C<$MESSAGE_SIZE_OVERHEAD>
 are the defaults that can be imported from the L<Kafka|Kafka> module.
 
 =item C<MaxNumberOfOffsets =E<gt> $max_number>
 
-Kafka is return up to C<$max_number> of offsets.
+Limit the number of offsets returned by Kafka.
 
 That is a non-negative integer.
 
@@ -280,7 +268,6 @@ sub new {
 
     my $self = bless {
         Connection          => undef,
-        RaiseError          => $DEFAULT_RAISE_ERROR,
         CorrelationId       => undef,
         ClientId            => undef,
         MaxWaitTime         => $DEFAULT_MAX_WAIT_TIME,
@@ -298,21 +285,20 @@ sub new {
     $self->{CorrelationId}  //= _get_CorrelationId;
     $self->{ClientId}       //= 'consumer';
 
-    if ( !defined _NONNEGINT( $self->RaiseError ) ) {
-        $self->{RaiseError} = $DEFAULT_RAISE_ERROR;
-        $self->_error( $ERROR_MISMATCH_ARGUMENT, 'RaiseError' );
-    }
-    elsif ( !_INSTANCE( $self->{Connection}, 'Kafka::Connection' ) )                    { $self->_error( $ERROR_MISMATCH_ARGUMENT, 'Connection' ); }
-    elsif ( !( defined( $self->{CorrelationId} ) && isint( $self->{CorrelationId} ) ) ) { $self->_error( $ERROR_MISMATCH_ARGUMENT, 'CorrelationId' ); }
-    elsif ( !( $self->{ClientId} eq q{} || defined( _STRING( $self->{ClientId} ) ) && !utf8::is_utf8( $self->{ClientId} ) ) )   { $self->_error( $ERROR_MISMATCH_ARGUMENT, 'ClientId' ); }
-    elsif ( !( defined( $self->{MaxWaitTime} ) && isint( $self->{MaxWaitTime} ) && $self->{MaxWaitTime} > 0 ) ) { $self->_error( $ERROR_MISMATCH_ARGUMENT, 'MaxWaitTime' ); }
-    elsif ( !defined( _NONNEGINT( $self->{MinBytes} ) ) )                               { $self->_error( $ERROR_MISMATCH_ARGUMENT, 'MinBytes' ); }
-    elsif ( !( _POSINT( $self->{MaxBytes} ) && $self->{MaxBytes} >= $MESSAGE_SIZE_OVERHEAD ) )      { $self->_error( $ERROR_MISMATCH_ARGUMENT, 'MaxBytes' ); }
-    elsif ( !defined( _POSINT( $self->{MaxNumberOfOffsets} ) ) )                        { $self->_error( $ERROR_MISMATCH_ARGUMENT, 'MaxNumberOfOffsets' ); }
-    else {
-        $self->_error( $ERROR_NO_ERROR )
-            if $self->last_error;
-    }
+    $self->_error( $ERROR_MISMATCH_ARGUMENT, 'Connection' )
+        unless _INSTANCE( $self->{Connection}, 'Kafka::Connection' );
+    $self->_error( $ERROR_MISMATCH_ARGUMENT, 'CorrelationId' )
+        unless defined( $self->{CorrelationId} ) && isint( $self->{CorrelationId} );
+    $self->_error( $ERROR_MISMATCH_ARGUMENT, 'ClientId' )
+        unless ( $self->{ClientId} eq q{} || defined( _STRING( $self->{ClientId} ) ) ) && !utf8::is_utf8( $self->{ClientId} );
+    $self->_error( $ERROR_MISMATCH_ARGUMENT, 'MaxWaitTime' )
+        unless defined( $self->{MaxWaitTime} ) && isint( $self->{MaxWaitTime} ) && $self->{MaxWaitTime} > 0;
+    $self->_error( $ERROR_MISMATCH_ARGUMENT, 'MinBytes' )
+        unless defined( _NONNEGINT( $self->{MinBytes} ) );
+    $self->_error( $ERROR_MISMATCH_ARGUMENT, 'MaxBytes' )
+        unless _POSINT( $self->{MaxBytes} ) && $self->{MaxBytes} >= $MESSAGE_SIZE_OVERHEAD;
+    $self->_error( $ERROR_MISMATCH_ARGUMENT, 'MaxNumberOfOffsets' )
+        unless defined( _POSINT( $self->{MaxNumberOfOffsets} ) );
 
     return $self;
 }
@@ -323,32 +309,17 @@ sub new {
 
 The following methods are defined for the C<Kafka::Consumer> class:
 
-=over 3
-
-=item *
-
-The arguments below B<offset>, B<max_size> or B<time>, B<max_number> are
-the additional information that might be encoded parameters of the messages
-we want to access.
-
-=back
-
-The following methods are defined for the C<Kafka::Consumer> class:
-
 =cut
 
 #-- public methods -------------------------------------------------------------
 
 =head3 C<fetch( $topic, $partition, $offset, $max_size )>
 
-Get a list of messages to consume one by one up C<$max_size> bytes.
+Get a list of messages to consume one by one up to C<$max_size> bytes.
 
-Returns the reference to array of the  L<Kafka::Message|Kafka::Message> class
-name elements.
-If there's an error,
-returns the reference to empty array if the C<RaiseError> is not true.
+Returns the reference to array of the L<Kafka::Message|Kafka::Message> objects.
 
-C<fetch()> takes arguments. The following arguments are currently recognized:
+C<fetch()> takes the following arguments:
 
 =over 3
 
@@ -362,15 +333,16 @@ The C<$partition> must be a non-negative integer.
 
 =item C<$offset>
 
-Offset in topic and partition to start from (64 bits).
+Offset in topic and partition to start from (64-bit integer).
 
 Optional. The argument must be a non-negative integer. The argument may be a
-L<Math::BigInt|Math::BigInt> integer on 32 bit system.
+L<Math::BigInt|Math::BigInt> integer on 32-bit system.
 
 =item C<$max_size>
 
-C<$max_size> is the maximum size of the message set to return. The argument
+C<$max_size> is the maximum size of the messages set to return. The argument
 must be a positive integer.
+
 The maximum size of a request limited by C<MAX_SOCKET_REQUEST_BYTES> that
 can be imported from L<Kafka|Kafka> module.
 
@@ -380,10 +352,14 @@ can be imported from L<Kafka|Kafka> module.
 sub fetch {
     my ( $self, $topic, $partition, $offset, $max_size ) = @_;
 
-    if    ( !( $topic eq q{} || defined( _STRING( $topic ) ) && !utf8::is_utf8( $topic ) ) )    { return $self->_error( $ERROR_MISMATCH_ARGUMENT, '$topic' ); }
-    elsif ( !( defined( $partition ) && isint( $partition ) ) )                                 { return $self->_error( $ERROR_MISMATCH_ARGUMENT, '$partition' ); }
-    elsif ( !( defined( $offset ) && ( _isbig( $offset ) && $offset >= 0 ) || defined( _NONNEGINT( $offset ) ) ) )   { return $self->_error( $ERROR_MISMATCH_ARGUMENT, '$offset' ); }
-    elsif ( defined( $max_size ) && !( _POSINT( $max_size ) && $max_size >= $MESSAGE_SIZE_OVERHEAD ) )  { return $self->_error( $ERROR_MISMATCH_ARGUMENT, '$max_size' ); }
+    $self->_error( $ERROR_MISMATCH_ARGUMENT, '$topic' )
+        unless defined( $topic ) && ( $topic eq q{} || defined( _STRING( $topic ) ) ) && !utf8::is_utf8( $topic );
+    $self->_error( $ERROR_MISMATCH_ARGUMENT, '$partition' )
+        unless defined( $partition ) && isint( $partition );
+    $self->_error( $ERROR_MISMATCH_ARGUMENT, '$offset' )
+        unless defined( $offset ) && ( ( _isbig( $offset ) && $offset >= 0 ) || defined( _NONNEGINT( $offset ) ) );
+    $self->_error( $ERROR_MISMATCH_ARGUMENT, '$max_size' )
+        unless ( !defined( $max_size ) || ( _POSINT( $max_size ) && $max_size >= $MESSAGE_SIZE_OVERHEAD ) );
 
     my $request = {
         ApiKey                              => $APIKEY_FETCH,
@@ -405,29 +381,19 @@ sub fetch {
         ],
     };
 
-    $self->_error( $ERROR_NO_ERROR )
-        if $self->last_error;
-
-    my $response;
-    try {
-        $response = $self->_fulfill_request( $request );
-    };
-    unless ( $response ) {
-        confess $self->last_error if $self->RaiseError;
-        return;
-    }
+    my $response = $self->{Connection}->receive_response_to_request( $request );
 
     my $messages = [];
     foreach my $received_topic ( @{ $response->{topics} } ) {
         $received_topic->{TopicName} eq $topic
-            or return $self->_error( $ERROR_TOPIC_DOES_NOT_MATCH, "'$topic' ne '".$received_topic->{TopicName}."'" );
+            or $self->_error( $ERROR_TOPIC_DOES_NOT_MATCH, "'$topic' ne '".$received_topic->{TopicName}."'" );
         foreach my $received_partition ( @{ $received_topic->{partitions} } ) {
             $received_partition->{Partition} == $partition
-                or return $self->_error( $ERROR_PARTITION_DOES_NOT_MATCH, "$partition != ".$received_partition->{Partition} );
+                or $self->_error( $ERROR_PARTITION_DOES_NOT_MATCH, "$partition != ".$received_partition->{Partition} );
             my $HighwaterMarkOffset = $received_partition->{HighwaterMarkOffset};
             foreach my $Message ( @{ $received_partition->{MessageSet} } ) {
-                my ( $offset, $next_offset, $message_error );
-                $offset = $Message->{Offset};
+                my $offset = $Message->{Offset};
+                my $next_offset;
                 if ( $BITS64 ) {
                     $next_offset += $offset + 1;
                 }
@@ -435,7 +401,7 @@ sub fetch {
                     $offset = Kafka::Int64::intsum( $offset, 0 );
                     $next_offset = Kafka::Int64::intsum( $offset, 1 );
                 }
-                $message_error = $Message->{Attributes} ? $ERROR{ $ERROR_COMPRESSED_PAYLOAD } : q{};
+                my $message_error = $Message->{Attributes} ? $ERROR{ $ERROR_COMPRESSED_PAYLOAD } : q{};
 
                 push( @$messages, Kafka::Message->new( {
                         Attributes          => $Message->{Attributes},
@@ -458,14 +424,12 @@ sub fetch {
 
 =head3 C<offsets( $topic, $partition, $time, $max_number )>
 
-Get a list of valid offsets up C<$max_number> before the given time.
+Get a list of valid offsets up to C<$max_number> before the given time.
 
-Returns reference to the offsets response array of the offset integers
+Returns reference to array of the offset integers
 (L<Math::BigInt|Math::BigInt> integers on 32 bit system).
-If there's an error,
-returns the reference to empty array if the C<RaiseError> is not true.
 
-C<offsets()> takes arguments. The following arguments are currently recognized:
+C<offsets()> takes the following arguments:
 
 =over 3
 
@@ -479,11 +443,10 @@ The C<$partition> must be a non-negative integer.
 
 =item C<$time>
 
-Used to ask for all messages before a certain time (ms).
-C<$time> is the timestamp of the offsets before this time - milliseconds since
-UNIX Epoch.
+Get offsets before the given time (in milliseconds since UNIX Epoch).
 
 The argument must be a positive number.
+
 The argument may be a L<Math::BigInt|Math::BigInt> integer on 32 bit
 system.
 
@@ -504,10 +467,14 @@ Optional. The argument must be a positive integer.
 sub offsets {
     my ( $self, $topic, $partition, $time, $max_number ) = @_;
 
-    if    ( !( $topic eq q{} || defined( _STRING( $topic ) ) && !utf8::is_utf8( $topic ) ) )    { return $self->_error( $ERROR_MISMATCH_ARGUMENT, '$topic' ); }
-    elsif ( !( defined( $partition ) && isint( $partition ) ) )                                 { return $self->_error( $ERROR_MISMATCH_ARGUMENT, '$partition' ); }
-    elsif ( !( defined( $time ) && ( _isbig( $time ) || isint( $time ) ) && $time >= $RECEIVE_EARLIEST_OFFSETS ) )   { return $self->_error( $ERROR_MISMATCH_ARGUMENT, '$time' ); }
-    elsif ( defined( $max_number ) && !_POSINT( $max_number ) )                                 { return $self->_error( $ERROR_MISMATCH_ARGUMENT, '$max_number' ); }
+    $self->_error( $ERROR_MISMATCH_ARGUMENT, '$topic' )
+        unless defined( $topic) && ( $topic eq q{} || defined( _STRING( $topic ) ) ) && !utf8::is_utf8( $topic );
+    $self->_error( $ERROR_MISMATCH_ARGUMENT, '$partition' )
+        unless defined( $partition ) && isint( $partition );
+    $self->_error( $ERROR_MISMATCH_ARGUMENT, '$time' )
+        unless defined( $time ) && ( _isbig( $time ) || isint( $time ) ) && $time >= $RECEIVE_EARLIEST_OFFSETS;
+    $self->_error( $ERROR_MISMATCH_ARGUMENT, '$max_number' )
+        unless !defined( $max_number ) || _POSINT( $max_number );
 
     my $request = {
         ApiKey                              => $APIKEY_OFFSET,
@@ -527,14 +494,7 @@ sub offsets {
         ],
     };
 
-    my $response;
-    try {
-        $response = $self->_fulfill_request( $request );
-    };
-    unless ( $response ) {
-        confess $self->last_error if $self->RaiseError;
-        return;
-    }
+    my $response = $self->{Connection}->receive_response_to_request( $request );
 
     my $offsets = [];
     foreach my $received_topic ( @{ $response->{topics} } ) {
@@ -543,9 +503,6 @@ sub offsets {
         }
     }
 
-    $self->_error( $ERROR_NO_ERROR )
-        if $self->last_error;
-
     return $offsets;
 }
 
@@ -553,35 +510,30 @@ sub offsets {
 
 #-- private methods ------------------------------------------------------------
 
+# Handler for errors
+sub _error {
+    my $self = shift;
+
+    Kafka::Exception::Consumer->throw( throw_args( @_ ) );
+}
+
 #-- Closes and cleans up -------------------------------------------------------
 
 1;
 
 __END__
 
-=head3 C<RaiseError>
-
-This method returns current value showing how errors are handled within Kafka module.
-If set to true, die() is dispatched when error during communication is detected.
-
-C<last_errorcode> and C<last_error> are diagnostic methods and can be used to get detailed
-error codes and messages for various cases: when server or the resource is not available,
-access to the resource was denied, etc.
-
-=head3 C<last_errorcode>
-
-Returns code of the last error.
-
-=head3 C<last_error>
-
-Returns an error message that contains information about the encountered failure.
-
 =head1 DIAGNOSTICS
 
-Review documentation of the L</RaiseError> method for additional information about possible errors.
+When error is detected, an exception, represented by object of C<Kafka::Exception::Consumer> class,
+is thrown (see L<Kafka::Exceptions|Kafka::Exceptions>).
 
-It's advised to always check L</last_errorcode> and more descriptive L</last_error> when
-L</RaiseError> is not set.
+L<code|Kafka::Exceptions/code> and a more descriptive L<message|Kafka::Exceptions/message> provide
+information about thrown exception. Consult documentation of the L<Kafka::Exceptions|Kafka::Exceptions>
+for the list of all available methods.
+
+Authors suggest using of L<Try::Tiny|Try::Tiny>'s C<try> and C<catch> to handle exceptions while
+working with Kafka module.
 
 =over 3
 
@@ -591,11 +543,11 @@ Invalid argument passed to a C<new> L<constructor|/CONSTRUCTOR> or other L<metho
 
 =item C<Can't send>
 
-Message can't be sent using L<Kafka::IO|Kafka::IO> object socket.
+Request cannot be sent.
 
 =item C<Can't recv>
 
-Message can't be received using L<Kafka::IO|Kafka::IO> object socket.
+Response cannot be received.
 
 =item C<Can't bind>
 
@@ -603,7 +555,7 @@ TCP connection can't be established on the given host and port.
 
 =item C<Can't get metadata>
 
-Failed to obtain metadata from kafka servers
+Failed to obtain metadata from Kafka servers.
 
 =item C<Leader not found>
 
@@ -623,8 +575,6 @@ Received metadata has incorrect internal structure.
 
 =back
 
-For more detailed error explanation call L</last_error> method of C<Kafka::Consumer> object.
-
 =head1 SEE ALSO
 
 The basic operation of the Kafka package modules:
@@ -637,7 +587,8 @@ L<Kafka::Producer|Kafka::Producer> - interface for producing client.
 
 L<Kafka::Consumer|Kafka::Consumer> - interface for consuming client.
 
-L<Kafka::Message|Kafka::Message> - interface to access Kafka message properties.
+L<Kafka::Message|Kafka::Message> - interface to access Kafka message
+properties.
 
 L<Kafka::Int64|Kafka::Int64> - functions to work with 64 bit elements of the
 protocol on 32 bit systems.
@@ -645,9 +596,11 @@ protocol on 32 bit systems.
 L<Kafka::Protocol|Kafka::Protocol> - functions to process messages in the
 Apache Kafka's Protocol.
 
-L<Kafka::IO|Kafka::IO> - low level interface for communication with Kafka server.
+L<Kafka::IO|Kafka::IO> - low-level interface for communication with Kafka server.
 
-L<Kafka::Internals|Kafka::Internals> - Internal constants and functions used
+L<Kafka::Exceptions|Kafka::Exceptions> - module designated to handle Kafka exceptions.
+
+L<Kafka::Internals|Kafka::Internals> - internal constants and functions used
 by several package modules.
 
 A wealth of detail about the Apache Kafka and the Kafka Protocol:
@@ -676,8 +629,7 @@ This package is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself. See I<perlartistic> at
 L<http://dev.perl.org/licenses/artistic.html>.
 
-This program is
-distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 PARTICULAR PURPOSE.
 
