@@ -7,7 +7,7 @@ protocol on 32 bit systems.
 
 =head1 VERSION
 
-This documentation refers to C<Kafka::Int64> version 0.800_3 .
+This documentation refers to C<Kafka::Int64> version 0.800_4 .
 
 =cut
 
@@ -21,7 +21,7 @@ use bigint; # this allows integers of practially any size at the cost of signifi
 
 # ENVIRONMENT ------------------------------------------------------------------
 
-our $VERSION = '0.800_3';
+our $VERSION = '0.800_4';
 
 use Exporter qw(
     import
@@ -41,6 +41,7 @@ use Kafka qw(
     %ERROR
     $ERROR_MISMATCH_ARGUMENT
 );
+use Kafka::Exceptions;
 
 #-- declarations ---------------------------------------------------------------
 
@@ -50,31 +51,43 @@ use Kafka qw(
     use strict;
     use warnings;
 
+    use Scalar::Util qw(
+        blessed
+    );
+    use Try::Tiny;
+
     use Kafka qw(
         $BITS64
     );
 
-    # Apache Kafka Protocol: FetchOffset, Time
+    try {
 
-    my $offset = 123;
+        # Apache Kafka Protocol: FetchOffset, Time
 
-    my $encoded = $BITS64 ?
-        pack( 'q>', $offset )
-        : Kafka::Int64::packq( $offset );
+        my $offset = 123;
 
-    my $response = chr( 0 ) x 8;
+        my $encoded = $BITS64
+            ? pack( 'q>', $offset )
+            : Kafka::Int64::packq( $offset );
 
-    $offset = $BITS64 ?
-        unpack( 'q>', substr( $response, 0, 8 ) )
-        : Kafka::Int64::unpackq( substr( $response, 0, 8 ) );
+        my $response = chr( 0 ) x 8;
 
-    my $next_offset;
-    if ( $BITS64 ) {
-        $next_offset = $offset + 1;
-    }
-    else {
-        $next_offset = Kafka::Int64::intsum( $offset, 1 );
-    }
+        $offset = $BITS64
+            ? unpack( 'q>', substr( $response, 0, 8 ) )
+            : Kafka::Int64::unpackq( substr( $response, 0, 8 ) );
+
+        my $next_offset = $BITS64
+            ? $offset + 1
+            : Kafka::Int64::intsum( $offset, 1 );
+
+    } catch {
+        if ( blessed( $_ ) && $_->isa( 'Kafka::Exception' ) ) {
+            warn 'Error: (', $_->code, ') ',  $_->message, "\n";
+            exit;
+        } else {
+            die $_;
+        }
+    };
 
 =head1 DESCRIPTION
 
@@ -93,7 +106,7 @@ The main features of the C<Kafka::Int64> module are:
 
 =item *
 
-Support for working with 64 bit elements of the Kafka Wire Format protocol
+Support for working with 64 bit elements of the Kafka protocol
 on 32 bit systems.
 
 =back
@@ -113,18 +126,17 @@ The following functions are available for the C<Kafka::Int64> module.
 Adds two numbers to emulate bigint adding 64-bit integers in 32-bit systems.
 
 The both arguments must be a number. That is, it is defined and Perl thinks
-it's a number. The first argument may be a L<Math::BigInt|Math::BigInt>
-integer.
+it's a number. Arguments may be a L<Math::BigInt|Math::BigInt>
+integers.
 
-Returns the value as a L<Math::BigInt|Math::BigInt> integer, or error will
-cause the program to halt (C<confess>) if the argument is not a valid number.
+Returns the value as a L<Math::BigInt|Math::BigInt> integer.
 
 =cut
 sub intsum {
     my ( $frst, $scnd ) = @_;
 
     my $ret = $frst + $scnd + 0;    # bigint coercion
-    confess $ERROR{ $ERROR_MISMATCH_ARGUMENT }
+    Kafka::Exception::Int64->throw( throw_args( $ERROR_MISMATCH_ARGUMENT, 'intsum' ) )
         if $ret->is_nan();
 
     return $ret;
@@ -141,8 +153,7 @@ it's a number. The argument may be a L<Math::BigInt|Math::BigInt> integer.
 The special values -1, -2 are allowed
 (C<$Kafka::RECEIVE_LATEST_OFFSET>, C<$Kafka::RECEIVE_EARLIEST_OFFSETS>).
 
-Returns the value as a packed binary string, or error will cause the program
-to halt (C<confess>) if the argument is a negative number.
+Returns the value as a packed binary string.
 
 =cut
 sub packq {
@@ -150,7 +161,7 @@ sub packq {
 
     if      ( $n == -1 )    { return pack q{C8}, ( 255 ) x 8; }
     elsif   ( $n == -2 )    { return pack q{C8}, ( 255 ) x 7, 254; }
-    elsif   ( $n < 0 )      { confess $ERROR{ $ERROR_MISMATCH_ARGUMENT }; }
+    elsif   ( $n < 0 )      { Kafka::Exception::Int64->throw( throw_args( $ERROR_MISMATCH_ARGUMENT, 'packq' ) ); }
 
     return pack q{H16}, substr( '00000000000000000000000000000000'.substr( ( $n + 0 )->as_hex(), 2 ), -16 );
 }
@@ -182,10 +193,19 @@ __END__
 
 =head1 DIAGNOSTICS
 
+When error is detected, an exception, represented by object of C<Kafka::Exception::Producer> class,
+is thrown (see L<Kafka::Exceptions|Kafka::Exceptions>).
+
 Any error L<functions|/FUNCTIONS> is FATAL.
 FATAL errors will cause the program to halt (C<confess>), since the
-problem is so severe that it would be dangerous to continue. (This can
-always be trapped with C<eval>.
+problem is so severe that it would be dangerous to continue.
+
+L<code|Kafka::Exceptions/code> and a more descriptive L<message|Kafka::Exceptions/message> provide
+information about thrown exception. Consult documentation of the L<Kafka::Exceptions|Kafka::Exceptions>
+for the list of all available methods.
+
+Authors suggest using of L<Try::Tiny|Try::Tiny>'s C<try> and C<catch> to handle exceptions while
+working with Kafka module.
 
 =over 3
 
@@ -217,9 +237,11 @@ protocol on 32 bit systems.
 L<Kafka::Protocol|Kafka::Protocol> - functions to process messages in the
 Apache Kafka's Protocol.
 
-L<Kafka::IO|Kafka::IO> - low level interface for communication with Kafka server.
+L<Kafka::IO|Kafka::IO> - low-level interface for communication with Kafka server.
 
-L<Kafka::Internals|Kafka::Internals> - Internal constants and functions used
+L<Kafka::Exceptions|Kafka::Exceptions> - module designated to handle Kafka exceptions.
+
+L<Kafka::Internals|Kafka::Internals> - internal constants and functions used
 by several package modules.
 
 A wealth of detail about the Apache Kafka and the Kafka Protocol:
