@@ -6,7 +6,7 @@ Kafka::Cluster - object interface to manage a test kafka cluster.
 
 =head1 VERSION
 
-This documentation refers to C<Kafka::Cluster> version 0.800_3 .
+This documentation refers to C<Kafka::Cluster> version 0.800_4 .
 
 =cut
 
@@ -18,7 +18,7 @@ use warnings;
 
 # ENVIRONMENT ------------------------------------------------------------------
 
-our $VERSION = '0.800_3';
+our $VERSION = '0.800_4';
 
 use Exporter qw(
     import
@@ -113,7 +113,7 @@ The ability to perform a query to necessary server cluster.
 
 =head2 EXPORT
 
-These variables are the constants and never change their values.
+The following constants are available for export
 
 =cut
 
@@ -153,7 +153,6 @@ my $start_dir;
 # protection against re-create the cluster object
 our $_used = 0;
 
-
 =head2 CONSTRUCTOR
 
 =head3 C<new>
@@ -258,10 +257,10 @@ sub new {
 
     opendir( my $dh, $kafka_data_dir )
         or confess "can't opendir $kafka_data_dir: $!";
-    while ( readdir( $dh ) ) {
-        next unless /^$KAFKA_LOGS_DIR_MASK/;
+    foreach my $file ( readdir( $dh ) ) {
+        next if $file !~ /^$KAFKA_LOGS_DIR_MASK/;
         ++$cluster_factor;
-        if ( !$cfg && -e ( $inifile = catfile( $kafka_data_dir, $_, $KAFKA_PROPERTIES_FILE ) ) ) {
+        if ( !$cfg && -e ( $inifile = catfile( $kafka_data_dir, $file, $KAFKA_PROPERTIES_FILE ) ) ) {
             if ( !( $cfg = Config::IniFiles->new(
                     -file       => $inifile,
                     -fallback   => $INI_SECTION,
@@ -547,7 +546,6 @@ sub start {
             my $io = Kafka::IO->new(
                 host       => 'localhost',
                 port       => $port,
-                RaiseError => 1,
             );
 
 # ***** A MetadataRequest example:
@@ -580,11 +578,11 @@ sub start {
             $$response .= $$tail;
             $io->close;
         } catch {
-            confess "Could not send control message: $_\n" unless $attempts;
+#            confess "Could not send control message: $_\n" unless $attempts;
             ++$error;
         };
 
-        last unless $error;
+#        last unless $error;
         sleep 1;
     }
 }
@@ -621,7 +619,6 @@ sub request {
     my $io = Kafka::IO->new(
         host       => 'localhost',
         port       => $port,
-        RaiseError => 1,
     );
 
     $io->send( pack( q{H*}, $bin_stream ) );
@@ -1119,9 +1116,11 @@ protocol on 32 bit systems.
 L<Kafka::Protocol|Kafka::Protocol> - functions to process messages in the
 Apache Kafka's Protocol.
 
-L<Kafka::IO|Kafka::IO> - low level interface for communication with Kafka server.
+L<Kafka::IO|Kafka::IO> - low-level interface for communication with Kafka server.
 
-L<Kafka::Internals|Kafka::Internals> - Internal constants and functions used
+L<Kafka::Exceptions|Kafka::Exceptions> - module designated to handle Kafka exceptions.
+
+L<Kafka::Internals|Kafka::Internals> - internal constants and functions used
 by several package modules.
 
 A wealth of detail about the Apache Kafka and the Kafka Protocol:
